/************************************************************************
     File:        main.cpp

     Modifier
                  Yu-Chi Lai, yu-chi@cs.wisc.edu
     
     Comment:     Load and modify image through the command line
   
     Platform:    Visio Studio.Net 2003
*************************************************************************/
#include <iostream>
#include "libtarga.h"

using namespace std;

int main(int argc, char** args)
{
	// Check whether the argument is four or not
	printf("The argc %d\n", argc);
	if(argc != 4){
		std::cerr << "Did not have enough input arguments which should be \n";
		std::cerr << "1. Source 2. Destination filenames 3. Size of the square\n";
		return 0;
	}

	// Load in the image
	unsigned char   *image_data;
  int		           width, height;
	char*            input = args[1];
	char*            output = args[2];
	int              radius = atoi(args[3]);

	// Check the input name
  if (!input) {
		std::cerr << "No input is given." << endl;
    return 0;
  }// if

	// Check the output name
  if (!output) {
		std::cerr << "No output is given." << endl;
    return NULL;
  }// if

	// Check the square size
	if(radius <= 0){
		std::cerr << "The size of square is negative." << endl;
    return NULL;
	}

	// Load in the image
	image_data = (unsigned char*)tga_load(input, &width, &height, TGA_TRUECOLOR_32);
	// Chech whether it succeed or not and print out necessary error message
  if (!image_data) {
		std::cerr << "TGA Error: %s\n", tga_error_string(tga_get_last_error());
	  width = height = 0;
	  return 0;
  }

	// Check the square size
	if(radius * 2 > width || radius * 2 > height){
		std::cerr << "The size of square bigger than image." << endl;
    return 0;
	}

	// Copy the image center
  int sourceCenterX          = width / 2;
  int sourceCenterY          = height / 2;

	// Go through the center of the square
  for (int yOffset = -radius; yOffset <= radius; ++yOffset)  {
		// Compute the index at Y coordinate
    int sourceY = sourceCenterY + yOffset;
		if(sourceY >=0 && sourceY < height) { // Check whether Y is over boundary
			for (int xOffset = -radius; xOffset <= radius; ++xOffset) {
				int sourceX = sourceCenterX + xOffset; // Compute the index at X coordinate
				if(sourceX >= 0 && sourceX < width) { // Check whether X is over boundary
					int index = (sourceY * width + sourceX) * 4; // Compute the index for this pixel

					image_data[index]   = 0;   // Change red to 0
					image_data[index+1] = 255; // Change green to 1
					image_data[index+2] = 0;   // Change blue to 0
					image_data[index+3] = 255; // Change alpha to 1
				} // end of if sourceX
	    } // end of for
		} // end of if sourceY
  } // end of for

  // Save the image
  if (!tga_write_raw(output, width, height, image_data, TGA_TRUECOLOR_32)) {
	  cout << "TGA Save Error: %s\n", tga_error_string(tga_get_last_error());
	  return 0;
  }

	printf("Finish image loading and processing\n");	
	return 0;

}

